// Onur G. Guleryuz 1995, 1996, 1997,
// University of Illinois at Urbana-Champaign,
// Princeton University,
// Polytechnic University.
#include <stdio.h>
#include "wav_basic.h"
#include "wav_filters_extern.h"
#include "alloc.h"
#include "wav_gen.h"

// 2D "in place" wavelet transform. Input data is in
// float **image and output is returned inside image.
//
// levs>=1 is the number of wavelet levels.
//
// Ni and Nj are the dimensions of the image.
// Ni and Nj should be divisible by 2^levs.
//
// shift_arr_row and shift_arr_col (each of dimension levs)
// contain the amount of shift (0 or 1)
// that should be applied to filters in each level for transforms over rows and columns.
// The use of shift_arr_row and shift_arr_col is mainly intended for 
// overcomplete filtering/transforms.
//
// forw != 0 for a forward transform (inverse is calculated otherwise).
//
// float *lp of dimension Nl contains the low pass wavelet filter.
// float *hp of dimension Nh contains the high pass wavelet filter.
//
void wav2d_inpl(float **image,int Ni,int Nj,int levs,char forw,int *shift_arr_row,int *shift_arr_col){

	int i,dimi,dimj;

	dimi=(forw)?Ni:(Ni>>(levs-1));
	dimj=(forw)?Nj:(Nj>>(levs-1));
	for(i=0;i<levs;i++) {
		if(forw){
			// Do the rows.
			// Adjust filter parameters for overcomplete filtering.
			begflp=shift_arr_row[i];   
			begfhp=-shift_arr_row[i];

			filt_n_dec_all_rows(image,dimi,dimj);

			// Now the columns.

			begflp=shift_arr_col[i];   
			begfhp=-shift_arr_col[i];

			filt_n_dec_all_cols(image,dimi,dimj);
		}
		else {

			// Rows.
			begilp=shift_arr_row[levs-1-i];
			begihp=-shift_arr_row[levs-1-i];

			ups_n_filt_all_rows(image,dimi,dimj,levs-1-i,shift_arr_row);

			// Columns.
			begilp=shift_arr_col[levs-1-i];
			begihp=-shift_arr_col[levs-1-i];

			ups_n_filt_all_cols(image,dimi,dimj,levs-1-i,shift_arr_col);
		}

		dimi=(forw)?(dimi>>1):(dimi<<1);
		dimj=(forw)?(dimj>>1):(dimj<<1);
	}
}


