// Onur G. Guleryuz 1995, 1996, 1997,
// University of Illinois at Urbana-Champaign,
// Princeton University,
// Polytechnic University.

#include "wav_basic.h"
#include "wav_filters_extern.h"
#include "alloc.h"
#include "wav_gen.h"

// Macro that sets filter parameters.
// Called via choose_filter().
#define set_filter_param(fname) \
{ \
	MFLP= fname ## _FLP; \
	MFHP= fname ## _FHP; \
	MILP= fname ## _ILP; \
	MIHP= fname ## _IHP; \
	\
	/* Filter size. */ \
	Nflp= fname ## _Nflp; \
	Nfhp= fname ## _Nfhp; \
	Nilp= fname ## _Nilp; \
	Nihp= fname ## _Nihp; \
	\
	/* Filter symmetry */ \
	PS= fname ## _PS; \
	\
	/* Filter starting points. */ \
	begflp=mfl= fname ## _begflp; \
	begfhp=mfh= fname ## _begfhp; \
	begilp=mil= fname ## _begilp; \
	begihp=mih= fname ## _begihp; \
}

// Set wavelet filter parameters depending on name
// and tap. See wav_filters.h and wav_filters_extern.h 
// This routine must be called before doing any transforms
// to select which wavelet bank to use.
void choose_filter(char name,int tap)
{
	switch (name) {
		case 'A':
			switch (tap) {
				default:
					set_filter_param(Antonini);
			}
			break;

		case 'H':
			switch (tap) {
				default:
					set_filter_param(Haar);
			}
			break;

		case 'N':
			switch (tap) {
				case -1:
					set_filter_param(Nick_flip);
					break;

				default:
					set_filter_param(Nick);
			}
			break;

		default:
			switch (tap) {
				case 9:
					set_filter_param(D79);
					break;

				default:
					set_filter_param(D8);
			}
	}
}

// 2D "in place" wavelet transform. Input data is in
// float **image and output is returned inside image.
//
// levs>=1 is the number of wavelet levels.
//
// Ni and Nj are the dimensions of the image.
// Ni and Nj should be divisible by 2^levs.
//
// shift_arr_row and shift_arr_col (each of dimension levs)
// contain the amount of shift (0 or 1)
// that should be applied to filters in each level for transforms over rows and columns.
// The use of shift_arr_row and shift_arr_col is mainly intended for 
// overcomplete filtering/transforms.
//
// forw != 0 for a forward transform (inverse is calculated otherwise).
//
// float *lp of dimension Nl contains the low pass wavelet filter.
// float *hp of dimension Nh contains the high pass wavelet filter.
//
void wav2d_inpl(float **image,int Ni,int Nj,int levs,float *lp,int Nl,
						float *hp,int Nh,char forw,int *shift_arr_row,int *shift_arr_col)

{
	int i,dimi,dimj;

	dimi=(forw)?Ni:(Ni>>(levs-1));
	dimj=(forw)?Nj:(Nj>>(levs-1));

	for(i=0;i<levs;i++) {

		if(forw) {

			// Do the rows.

			// Adjust filter parameters for overcomplete filtering.
			begflp=mfl+shift_arr_row[i];   
			begfhp=mfh-shift_arr_row[i];

			filt_n_dec_all_rows(image,dimi,dimj,lp,Nl,hp,Nh);

			// Now the columns.

			begflp=mfl+shift_arr_col[i];   
			begfhp=mfh-shift_arr_col[i];

			filt_n_dec_all_cols(image,dimi,dimj,lp,Nl,hp,Nh);
		}
		else {

			// Rows.
			begilp=mil+shift_arr_row[levs-1-i];
			begihp=mih-shift_arr_row[levs-1-i];

			ups_n_filt_all_rows(image,dimi,dimj,lp,Nl,hp,Nh,levs-1-i,shift_arr_row);

			// Columns.
			begilp=mil+shift_arr_col[levs-1-i];
			begihp=mih-shift_arr_col[levs-1-i];

			ups_n_filt_all_cols(image,dimi,dimj,lp,Nl,hp,Nh,levs-1-i,shift_arr_col);
		}

		dimi=(forw)?(dimi>>1):(dimi<<1);
		dimj=(forw)?(dimj>>1):(dimj<<1);
	}
}


