// Onur G. Guleryuz 1995, 1996, 1997,
// University of Illinois at Urbana-Champaign,
// Princeton University,
// Polytechnic University.
#include <stdio.h>
#include <stdlib.h>
#include "wavelets1.h"
#include "test.h"
//  40   10241024
#define check_ptr(ptr,fn) \
	if((ptr)==NULL) { \
		printf("%10s: NULL pointer\n",(char *)(fn)); \
		perror((char *)(fn)); \
		exit(1); \
	}

#ifdef __TEST__
double timesum=0.; int cntr=1; bool firsttime=1;//   
#endif

float MLP[2] ={(float)0.70711,(float)0.70711};
float MHP[2] ={(float)0.70711,(float)-0.70711};
int begilp=0, begihp=0, begflp=0, begfhp=0;

WImage alloc_ima(int N,int M,char zero){
	WImage mymat;
	if(!zero)
		mymat = (WImage) malloc(N*M*sizeof(float));
	else
		mymat = (WImage) calloc(N*M, sizeof(float));
	return(mymat);
}

inline void free_ima(WImage a){
	free((void *)a);
}

float *allocate_1d_float(int N,char zero){
	float *arr;
	if(!zero)
		arr=(float *)malloc(N*sizeof(float));
	else
		arr=(float *)calloc(N,sizeof(float));
	check_ptr(arr,"allocate_1d_float");
	return(arr);
}

void extend_periodic(float *data,int N){
	data[-1] = data[N-1];
	data[-2] = data[N-2];
	data[N] = data[0];
	data[N+1] = data[1];
}

int filter_n_decimate(float *data,float *coef,int N,float *filter,int beg){
	int i,k;
	k=0;
	for(i=beg;i<N+beg;i+=2){
		coef[k] = data[i]*filter[0] + data[i-1]*filter[1];
		k++;
	}
	return(k);
}

void upsample_n_filter(float *coef,float *data,int N,float *filter,int beg){
	int i,l,n,p,fbeg;
	fbeg=1-beg;
	for(i=0;i<N;i++){
		l=0;
		n=(i+fbeg)%2;
		p=(i+fbeg)/2;
		data[i] += coef[p-l] * filter[n];
		l++;
	}
}

void filt_n_dec_all_rows(WImage image,int Ni,int Nj,int Wd){
	int i,j,ofs;
	float *data, *ptr, *ptr0, *ptr00;
	data=allocate_1d_float((Nj+4),0);
	data+=2;
	ofs=Nj>>1;
	ptr00 = image;
	for(i=0;i<Ni;i++){
		ptr = data; ptr0 = ptr00;
		for(j=0;j<Nj;j++)
			*ptr++  = *ptr0++;
		extend_periodic(data,Nj);
		filter_n_decimate(data,ptr00,Nj,MLP,begflp);
		filter_n_decimate(data,ptr00+ofs,Nj,MHP,begfhp);
		ptr00 += Wd;
	}
	data-=2;
	free((void *)data);
}

void filt_n_dec_all_cols(WImage image,int Ni,int Nj,int Wd){
	int i,j,ofs;
	float *data,*data2, *ptr, *ptr0;
	data=allocate_1d_float((Ni+4),0);
	data+=2;
	data2=allocate_1d_float(Ni,0);
	ofs=Ni>>1;
	for(j=0;j<Nj;j++){
		ptr = data; ptr0 = &image[j];
		for(i=0;i<Ni;i++){
			*ptr++  = *ptr0;
			ptr0 += Wd;
		}
		extend_periodic(data,Ni);
		filter_n_decimate(data,data2,Ni,MLP,begflp);
		filter_n_decimate(data,data2+ofs,Ni,MHP,begfhp);
		ptr = data2; ptr0 = &image[j];
		for(i=0;i<Ni;i++){
			*ptr0 = *ptr++;
			ptr0 += Wd;
		}
	}
	data-=2;
	free((void *)data);
	free((void *)data2);
}

void ups_n_filt_all_rows(WImage image,int Ni,int Nj,int lev,int *shift_arr, int Wd){
	int i,j,ofs;
	float *data1,*data2, *ptr, *ptr1, *ptr2, *ptr0;
	ofs=Nj>>1;
	data1=allocate_1d_float((ofs+4),0);
	data2=allocate_1d_float((ofs+4),0);
	data1+=2;data2+=2;
	ptr0 = image;
	for(i=0;i<Ni;i++) {	
		ptr1 = data1; ptr2 = data2; ptr = ptr0;
		for(j=0;j<ofs;j++, ptr++){
			*ptr1++ = *ptr; *ptr = 0;
			*ptr2++ = ptr[ofs]; ptr[ofs] = 0;
		}
		extend_periodic(data1,ofs);	
		extend_periodic(data2,ofs);	
		upsample_n_filter(data1,ptr0,Nj,MLP,begilp);
		upsample_n_filter(data2,ptr0,Nj,MHP,begihp);
		ptr0 += Wd;
	}
	data1-=2;data2-=2;
	free((void *)data1);
	free((void *)data2);
}

void ups_n_filt_all_cols(WImage image,int Ni,int Nj,int lev,int *shift_arr, int Wd){
	int i,j,k,ofs;
	float *data1,*data2,*data3, *ptr, *ptr1;
	ofs=Ni>>1;
	data1=allocate_1d_float((ofs+4),0);
	data2=allocate_1d_float((ofs+4),0);
	data1+=2;data2+=2;
	data3=allocate_1d_float(Ni,0);
	for(j=0;j<Nj;j++){	
		for(i=0;i<ofs;i++) {
			k=i+ofs;
			data1[i]=image[i*Wd+j];
			data2[i]=image[k*Wd+j];
			data3[i]=data3[k]=0;
		}
		extend_periodic(data1,ofs);	
		extend_periodic(data2,ofs);	
		upsample_n_filter(data1,data3,Ni,MLP,begilp);
		upsample_n_filter(data2,data3,Ni,MHP,begihp);
		ptr = &image[j];
		ptr1 = data3;
		for(i=0;i<Ni;i++){
			*ptr = *ptr1++;
			ptr += Wd;
		}
	}
	data1-=2;data2-=2;
	free((void *)data1);
	free((void *)data2);
	free((void *)data3);
}

void wav2d_inpl(WImage image,int Ni,int Nj,int levs,char forw,int *shift_arr_row,int *shift_arr_col){
	int i,dimi,dimj;
	#ifdef __TEST__
	double t0 = dtime();
	#endif
	dimi=(forw)?Ni:(Ni>>(levs-1));
	dimj=(forw)?Nj:(Nj>>(levs-1));
	for(i=0;i<levs;i++){
		if(forw){
			begflp=shift_arr_row[i];   
			begfhp=-shift_arr_row[i];
			filt_n_dec_all_rows(image,dimi,dimj,Ni);
			begflp=shift_arr_col[i];   
			begfhp=-shift_arr_col[i];
			filt_n_dec_all_cols(image,dimi,dimj,Ni);
		}
		else{
			begilp=shift_arr_row[levs-1-i];
			begihp=-shift_arr_row[levs-1-i];
			ups_n_filt_all_rows(image,dimi,dimj,levs-1-i,shift_arr_row,Ni);
			begilp=shift_arr_col[levs-1-i];
			begihp=-shift_arr_col[levs-1-i];
			ups_n_filt_all_cols(image,dimi,dimj,levs-1-i,shift_arr_col,Ni);
		}
		dimi=(forw)?(dimi>>1):(dimi<<1);
		dimj=(forw)?(dimj>>1):(dimj<<1);
	}
	#ifdef __TEST__
	{
	double dt = dtime() - t0;
	if(!firsttime) timesum += dt;
	fprintf(stderr, "\nmkWavedec(), time: %.6f (aver %.6f)\n", dt, timesum/cntr++);
	if(firsttime){cntr = 1; firsttime = 0;}
	}
	#endif
}

